//-*-c++-*------------------------------------------------------------
//
// File name : bioArithAbs.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Tue May  5 10:56:09 2009
//
//--------------------------------------------------------------------

#include <sstream>

#include "patDisplay.h"
#include "patErrNullPointer.h"
#include "patErrMiscError.h"

#include "bioLiteralRepository.h"
#include "bioArithAbs.h"
#include "bioArithConstant.h"
#include "bioExpressionRepository.h" 

bioArithAbs::bioArithAbs(bioExpressionRepository* rep, 
			 patULong par,
                         patULong c,
			 patError*& err) : bioArithUnaryExpression(rep,par,c,err)
{
}

bioArithAbs::~bioArithAbs() {}

patString bioArithAbs::getOperatorName() const {
  return ("abs") ;
}

patReal bioArithAbs::getValue(patBoolean prepareGradient, patError*& err)  {
  if (child == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patReal() ;
  }
  patReal result = child->getValue(prepareGradient,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patReal();
  }
  return ((result>=0)?result:-result) ;
}


bioExpression* bioArithAbs::getDerivative(patULong aLiteralId, patError*& err) const {
  if (dependsOf(aLiteralId)) {
    err = new patErrMiscError("No derivative for absolute value") ;
    WARNING(err->describe()) ;
    return NULL ;
  }
  else {
    bioExpression* result = new bioArithConstant(theRepository,NULL,0.0) ;
    return result ;
  }
}

bioArithAbs* bioArithAbs::getDeepCopy(bioExpressionRepository* rep,
				      patError*& err) const {
  if (child == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return NULL ;
  }
  bioExpression* leftClone(NULL) ;
  if (child != NULL) {
    leftClone = child->getDeepCopy(rep,err) ;
  }

  bioArithAbs* theNode = 
    new bioArithAbs(rep,patBadId,leftClone->getId(),err) ;
  if (err != NULL) {
    WARNING(err->describe());
    return NULL ;
  }
  return theNode ;
}


bioArithAbs* bioArithAbs::getShallowCopy(bioExpressionRepository* rep,
				      patError*& err) const {
  bioArithAbs* theNode = 
    new bioArithAbs(rep,patBadId,childId,err) ;
  if (err != NULL) {
    WARNING(err->describe());
    return NULL ;
  }
  return theNode ;
}


patString bioArithAbs::getCppCode(patString prefix, patError*& err) {
  // if (codeMustBeGenerated) {
  // if (child == NULL) {
  //   err = new patErrNullPointer("bioExpression") ;
  //   WARNING(err->describe()) ;
  //   return patString();
  // }
  
  // patString vLeft = child->getCppCode(prefix, err) ;
  // if (err != NULL) {
  //   WARNING(err->describe()) ;
  //   return patString();
  // }
  // stringstream str ;
  // str << vLeft << endl ;
  // str << getCppResultDeclaration(prefix) << " = patAbs(" << child->getCppResult(prefix) << ");" << endl ;
  // //  str << "DEBUG_MESSAGE(\"" << getCppResult(prefix) << "= \" << " << getCppResult(prefix) << ");" << endl ;
  // codeMustBeGenerated = patFALSE ;
  // return patString(str.str()) ;
  // }
  // else {
  //   return patString() ;
  // }

  return patString() ;
}

patString bioArithAbs::getSimpleCppCode(patError*& err) {
  // if (child == NULL) {
  //   err = new patErrNullPointer("bioExpression") ;
  //   WARNING(err->describe()) ;
  //   return patString();
  // }
  // patString vLeft = child->getSimpleCppCode(err) ;
  // if (err != NULL) {
  //   WARNING(err->describe()) ;
  //   return patString();
  // }
  // stringstream str ;
  // str << "patAbs(" << vLeft << ")" ;
  // return patString(str.str()) ;
  return patString() ;
}

patString bioArithAbs::getExpressionString() const {
  // stringstream str ;
  // str << "$A" ;
  // if (child != NULL) {
  //   str << '{' << child->getExpressionString() << '}' ;
  // }
  // return patString(str.str()) ;
  return patString() ;
}




bioFunctionAndDerivatives* bioArithAbs::getNumericalFunctionAndGradient(vector<patULong> literalIds,
							   patError*& err) {
  if (result.empty()) {
    result.resize(literalIds.size(),0.0) ;
  }
  result.theFunction = getValue(patFALSE,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL ;
  }
  return &result ;
}
