//-*-c++-*------------------------------------------------------------
//
// File name : bioArithGaussHermite.cc
// Author :    Michel Bierlaire
// Date :      Sat May 21 19:25:23 2011
//
//--------------------------------------------------------------------

#include "bioArithGaussHermite.h"
#include "bioExpression.h"
#include "bioLiteralRepository.h"
#include "patDisplay.h"
bioArithGaussHermite::bioArithGaussHermite(bioExpression* e,
					   vector<patULong> derivl,
					   patULong l,
					   patBoolean wg) : 
  withGradient(wg),
  theExpression(e),
  derivLiteralIds(derivl),
  literalId(l) {
  
}

vector<patReal> bioArithGaussHermite::getValue(patReal x, patError*& err) {
  bioLiteralRepository::the()->setRandomVariableValue(x, 
						      literalId, 
						      getThreadId(), 
						      err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return vector<patReal>() ;
  }
  vector<patReal> result ;
  if (withGradient) {
    bioFunctionAndDerivatives* fg = theExpression->getNumericalFunctionAndGradient(derivLiteralIds,err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return vector<patReal>() ;
    }
    result.push_back(fg->theFunction) ;
    for (patULong i = 0 ; i < fg->theGradient.size() ; ++i) {
      result.push_back(fg->theGradient[i]) ;
    }
  }
  else{
    patReal r = theExpression->getValue(patFALSE,err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return vector<patReal>() ;
    }
    result.push_back(r) ;
  }
  return result ;
}

patULong bioArithGaussHermite::getSize() const {
  if (withGradient) {
    return 1 + derivLiteralIds.size() ;
  }
  else {
    return 1 ;
  }
}

patULong bioArithGaussHermite::getThreadId() const {
  if (theExpression == NULL) {
    return patBadId ;
  }
  return theExpression->getThreadId() ;
}
