//-*-c++-*------------------------------------------------------------
//
// File name : bioArithLogit.h
// Author :    Michel Bierlaire
// Date :      Sat May 21 11:21:22 2011
//
//--------------------------------------------------------------------

#ifndef bioArithLogit_h
#define bioArithLogit_h


#include "bioExpression.h"

/*!
We have a dictionary of utilities, mapping values to utilities.
*/
class bioArithLogit : public bioExpression {

public:
  
  bioArithLogit(bioExpressionRepository* rep,
		patULong par,
		patULong ind,
		map<patULong,patULong> aDict,
		map<patULong,patULong> avDict,
		patError*& err) ;
  ~bioArithLogit() ;

public:
  virtual patString getOperatorName() const;
  virtual patReal getValue(patBoolean prepareGradient, patError*& err)  ;
  virtual bioExpression* getDerivative(patULong aLiteralId, patError*& err) const ;
  virtual bioArithLogit* getDeepCopy(bioExpressionRepository* rep,
				     patError*& err) const ;
  virtual bioArithLogit* getShallowCopy(bioExpressionRepository* rep,
				     patError*& err) const ;
  virtual patString getCppCode(patString prefix, patError*& err) ;
  virtual patString getSimpleCppCode(patError*& err) ;
  virtual patString getExpression(patError*& err) const ;
  virtual patBoolean isSimple() const ;
  virtual patBoolean dependsOf(patULong aLiteralId) const ;
  virtual void forceCodeGeneration() ;
  virtual void simplifyZeros(patError*& err) ;
  virtual patULong getNumberOfOperations() const ;
  patULong lastIndex() const ;

  virtual patString getExpressionString() const ;
  patBoolean containsAnIntegral() const ;
  patBoolean containsAnIterator() const ;
  patBoolean containsAnIteratorOnRows() const ;
  patBoolean containsASequence() const ;
  void generateGslFunctions(ostream& str_h,patString prefix,patError*& err) ;
  virtual patString getIncludeStatements(patString prefix) const  ;
  virtual void setVariables(const patVariables* x) ;
  //  virtual void setDraws(const vector< vector<patReal> >* d) ;
  virtual void setDraws(patReal** d) ;
  virtual void setSample(const bioSample* s) ;
  virtual void setCurrentSpan(bioIteratorSpan aSpan) ;
  virtual void setThreadSpan(bioIteratorSpan aSpan) ;
  virtual void collectExpressionIds(set<patULong>* s) const ;
  
  virtual bioFunctionAndDerivatives* getNumericalFunctionAndGradient(vector<patULong> literalIds, patError*& err) ;

  virtual patString check(patError*& err) const  ;

private:
  patReal index ;
  bioExpression* indexCalculation ;
  patULong indexCalculationId ;
  // First argument: user ID
  // Second argument: pointer to the expression repository
  map<patULong,patULong> theDictionaryIds ;
  map<patULong,patULong> theAvailDictIds ;
  map<patULong,bioExpression*> theDictionary ;
  map<patULong,bioExpression*> theAvailDict ;
  vector<patULong> theCompositeLiteralsIds ;
  vector<bioFunctionAndDerivatives*> Vig ;


};

#endif
