//-*-c++-*------------------------------------------------------------
//
// File name : bioArithUnaryMinus.cc
// Author :    Michel Bierlaire and Mamy Fetiarison
// Date :      Fri May 15 09:41:41 2009
//
//--------------------------------------------------------------------

#include "bioArithUnaryMinus.h"
#include "patErrNullPointer.h"
#include "patErrMiscError.h"
#include "patDisplay.h"
#include "bioArithConstant.h"
#include "bioExpressionRepository.h"

bioArithUnaryMinus::bioArithUnaryMinus(bioExpressionRepository* rep,
				       patULong par,
                                       patULong left,
				       patError*& err) 
  : bioArithUnaryExpression(rep, par,left,err) {

}

bioArithUnaryMinus::~bioArithUnaryMinus() {
}

patString bioArithUnaryMinus::getOperatorName() const {
  return("-") ;
}

patReal bioArithUnaryMinus::getValue(patBoolean prepareGradient,patError*& err)  {
  
  if (child == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patReal();
  }
  
  patReal result = child->getValue(prepareGradient, err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patReal();
  }
  return -result ;
}

bioExpression* bioArithUnaryMinus::getDerivative(patULong aLiteralId, 
						 patError*& err) const {
  if (child == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return NULL;
  }
  
  bioExpression* leftResult = child->getDerivative(aLiteralId,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL;
  }

  bioExpression* result = new bioArithUnaryMinus(theRepository,patBadId,leftResult->getId(),err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL ;
  }
 return result ;
}

bioArithUnaryMinus* bioArithUnaryMinus::getDeepCopy(bioExpressionRepository* rep,
						    patError*& err) const {
  bioExpression* leftClone(NULL) ;
  if (child != NULL) {
    leftClone = child->getDeepCopy(rep,err) ;
  }
  bioArithUnaryMinus* theNode = 
    new bioArithUnaryMinus(rep,patBadId,leftClone->getId(),err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL ;
  }
  return theNode ;
}


bioArithUnaryMinus* bioArithUnaryMinus::getShallowCopy(bioExpressionRepository* rep,
						    patError*& err) const {
  bioArithUnaryMinus* theNode = 
    new bioArithUnaryMinus(rep,patBadId,child->getId(),err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL ;
  }
  return theNode ;
}

patString bioArithUnaryMinus::getCppCode(patString prefix,patError*& err) {
  if (codeMustBeGenerated) {
    if (child == NULL) {
      err = new patErrNullPointer("bioExpression") ;
      WARNING(err->describe()) ;
      return patString();
    }
    patString vLeft = child->getCppCode(prefix, err) ;
    if (err != NULL) {
      WARNING(err->describe()) ;
      return patString();
    }
    stringstream str ;
    str << vLeft << endl ;
    str << getCppResultDeclaration(prefix) << " = - ( " << child->getCppResult(prefix) << "  ) ;" << endl ;
    //    str << "DEBUG_MESSAGE(\"" << getCppResult(prefix) << "= \" << " << getCppResult(prefix) << ")" ;
    codeMustBeGenerated = patFALSE ;
    return patString(str.str()) ;
  }
  else {
    return patString() ;
  }
}

patString bioArithUnaryMinus::getSimpleCppCode(patError*& err) {
  if (child == NULL) {
    err = new patErrNullPointer("bioExpression") ;
    WARNING(err->describe()) ;
    return patString();
  }
  patString vLeft = child->getSimpleCppCode(err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return patString();
  }
  stringstream str ;
  str << "-(" << vLeft << ")" ;
  return patString(str.str()) ;
}



patString bioArithUnaryMinus::getExpressionString() const {
  stringstream str ;
  str << "$M" ;
  if (child != NULL) {
    str << '{' << child->getExpressionString() << '}' ;
  }
  return patString(str.str()) ;
}


bioFunctionAndDerivatives* bioArithUnaryMinus::getNumericalFunctionAndGradient(vector<patULong> literalIds,  patError*& err) {
  if (result.empty()) {
    result.resize(literalIds.size()) ;
  }
  bioFunctionAndDerivatives* c = child->getNumericalFunctionAndGradient(literalIds,err) ;
  if (err != NULL) {
    WARNING(err->describe()) ;
    return NULL ;
  }
  result.theFunction =  - c->theFunction ;
  for (patULong i = 0 ; i < literalIds.size() ; ++i) {
    result.theGradient[i] = - c->theGradient[i] ;
  }
  return &result ;
}
