//-*-c++-*------------------------------------------------------------
//
// File name : patSolvOpt.h
// Author :    \URL[Michel Bierlaire]{http://rosowww.epfl.ch/mbi}
// Date :      Tue Feb  5 11:38:44 2002
//
//--------------------------------------------------------------------

#ifndef patSolvOpt_h
#define patSolvOpt_h 

/**
@doc This class defines a C++ interface to the SolvOpt routine, by Alexcei Kuntsevich and Franz Kappel. \URL{http://www.kfunigraz.ac.at/imawww/kuntsevich/solvopt}
   @author \URL[Michel Bierlaire]{http://rosowww.epfl.ch/mbi}, EPFL (Tue Feb  5 11:38:55 2002) 

*/


#include "patError.h"
#include "trNonLinearAlgo.h"
#include "solvoptParameters.h"

class patNonLinearProblem ;

class patSolvOpt : public trNonLinearAlgo {

  friend double ObjectFunctionValue(double x[]) ;
  friend void ObjectFunctionGradient(double x[], double g[]) ;
  friend double MaxResidual(double x[]) ;
  friend void GradMaxResConstr(double x[], double gc[]) ;

public:
  typedef enum {patNonLinEq,
		patLinEq,
		patNonLinIneq,
		patLinIneq,
		patUpperBound,
		patLowerBound} patMaxConstraintType ;
public:
  /**
   */
  patSolvOpt(solvoptParameters theParameters, 
	     patNonLinearProblem* aProblem = NULL) ;
  /**
   */
  virtual ~patSolvOpt() ;
  /**
   */
  void setProblem(patNonLinearProblem* aProblem) ;
  /**
   */
  patNonLinearProblem* getProblem() ;

  /**
   */
  void defineStartingPoint(const patVariables& x0) ;

  /**
   */
  patVariables getStartingPoint() ;
  /**
   */
  patVariables getSolution(patError*& err) ;

  /**
   */
  patReal getValueSolution(patError*& err) ;

  /**
   */
  patString getName() ;

  /**
     @return Diagnostic from cfsqp
   */
  patString run(patError*& err) ;
  /**
     @return number of iterations. If there is any error, 0 is returned. 
   */
  patULong nbrIter() ;

  /**
   */
   patVariables getLowerBoundsLambda() ;
  /**
   */
   patVariables getUpperBoundsLambda() ;

private:
  patVariables startingPoint ;
  patVariables solution ;
  solvoptParameters theParameters ;
  patULong nIter ;
  
};

/**
 @doc User functions required by solvopt
 */
  double ObjectFunctionValue(double x[]) ;
/**
 @doc User functions required by solvopt
 */
  void ObjectFunctionGradient(double x[], double g[]) ;
/**
 @doc User functions required by solvopt
 */
  double MaxResidual(double x[]) ;
/**
 @doc User functions required by solvopt
 */
  void GradMaxResConstr(double x[], double gc[]) ;

#endif
